import math
import os
import arcpy


class Toolbox(object):
    def __init__(self):
        """Define the toolbox (the name of the toolbox is the name of the
        .pyt file)."""
        self.label = "Create Transect"
        self.alias = "transect"

        # List of tool classes associated with this toolbox
        self.tools = [CreateLongestTransect]


class CreateLongestTransect(object):
    def __init__(self):
        """Define the tool (tool name is the name of the class)."""
        self.label = "Create Longest Transect"
        self.description = \
           "Creates a transect that crosses through the centroid of " +\
           "each feature."
        self.canRunInBackground = True

    def getParameterInfo(self):
        """Define parameter definitions"""
        in_lines = arcpy.Parameter(
            displayName="Input polygon features",
            name="in_poly_features",
            datatype="Feature Layer",
            parameterType="Required",
            direction="Input")
        in_lines.filter.list = ['Polygon']

        out_points = arcpy.Parameter(
            displayName="Output line feature class",
            name="out_line_features",
            datatype="Feature Class",
            parameterType="Required",
            direction="Output")

        evaluation_line_count = arcpy.Parameter(
            displayName="Evaluation line count",
            name="evaluation_line_count",
            datatype="Long",
            parameterType="Required",
            direction="Input")
        evaluation_line_count.value = 180

        return [in_lines, out_points, evaluation_line_count]

    def isLicensed(self):
        """Set whether tool is licensed to execute."""
        return True

    def updateParameters(self, parameters):

        parameters[1].parameterDependencies = [parameters[0].name]
        parameters[1].schema.clone = True
        parameters[1].schema.geometryTypeRule = "AsSpecified"
        parameters[1].schema.geometryType = "Polyline"
        parameters[1].schema.fieldsRule = "FirstDependencyFIDs"
        parameters[1].schema.fieldsRule = "None"

        id_field = arcpy.Field()
        id_field.name = "FID_1"
        id_field.type = "Integer"

        parameters[1].schema.additionalFields = [id_field]
        return

    def updateMessages(self, parameters):
        """Modify the messages created by internal validation for each tool
        parameter.  This method is called after internal validation."""
        err_value = "Value must be greater than zero."

        if parameters[2].value < 1:
            parameters[2].setErrorMessage(err_value)

        return

    def execute(self, parameters, messages):
        """The source code of the tool."""
        in_fc = parameters[0].valueAsText
        out_fc = parameters[1].valueAsText
        evaluation_line_count = parameters[2].value

        warning_empty = "Skipping OID {0}: feature is a null geometry"
        warning_centroid = "Skipping OID {0}: centroid falls outside feature"

        desc = arcpy.Describe(in_fc)

        # Create output feature class
        arcpy.CreateFeatureclass_management(
            os.path.dirname(out_fc),
            os.path.basename(out_fc),
            geometry_type="POLYLINE",
            spatial_reference=desc.spatialReference)

        # Add a field to transfer FID from input
        fid_name = "FID_1"
        arcpy.AddField_management(out_fc, fid_name, "LONG")

        with arcpy.da.SearchCursor(
                in_fc, ['SHAPE@', 'SHAPE@LENGTH', 'OID@']) as s_cursor:
            with arcpy.da.InsertCursor(
                    out_fc, ['SHAPE@', fid_name]) as i_cursor:
                for row in s_cursor:
                    longest_transect_distance = 0.0
                    poly = row[0]
                    if poly:  # skip null geometries
                        centroid = poly.trueCentroid
                        if poly.contains(centroid):  # skip if centroid outside
                            distance = row[1] / 2.0  # probable maximum distance
                            for cnt in xrange(0, evaluation_line_count):
                                angle = float(cnt) / evaluation_line_count * 180.0
                                cos = math.cos(math.radians(angle))
                                sin = math.sin(math.radians(angle))
                                polyline = arcpy.Polyline(
                                    arcpy.Array([
                                        arcpy.Point(
                                            centroid.X - cos * distance,
                                            centroid.Y - sin * distance),
                                        arcpy.Point(
                                            centroid.X + cos * distance,
                                            centroid.Y + sin * distance)]),
                                    desc.spatialReference)
                                intersect_line = poly.intersect(polyline, 2)
                                if intersect_line.partCount == 1:
                                    if intersect_line.length > longest_transect_distance:
                                        longest_transect_distance = intersect_line.length
                                        longest_line = intersect_line
                                else:
                                    for part in intersect_line:
                                        line_part = arcpy.Polyline(
                                            part, desc.spatialReference)
                                        if centroid.within(line_part):
                                            if line_part.length > longest_transect_distance:
                                                longest_transect_distance = line_part.length
                                                longest_line = line_part
                            i_cursor.insertRow([longest_line, row[2]])
                        else:
                            arcpy.AddWarning(warning_centroid.format(row[2]))
                    else:
                        arcpy.AddWarning(warning_empty.format(row[2]))


        return
